/*
 * quickie to test interaction (if any) between TT & STe palette registers on a TT
 *
 * syntax:	ttpal <testreg1> ...
 *
 * method:	write specified STe register(s) and read STe/TT registers
 *			write specified TT register(s) and read STe/TT registers
 */
#include <stdio.h>
#include <stdlib.h>

#include <cookie.h>
#include <osbind.h>
#include <portab.h>

#define VERSION	"0.9"

#define TT_VIDEO 		0x00020000

#define STE_YELLOW		0x0f70		/* use STe palette bit 0 */
#define TT_BLUE			0x000e

long cookie = 0L;
int testreg = 0;

volatile WORD *ste_palette = (volatile WORD *)0x00ff8240L;
volatile WORD *tt_palette = (volatile WORD *)0x00ff8400L;

void testpal(char *name,volatile WORD *palette,WORD newcolour);
long test(void);
void quit(int rc);

int main(int argc,char **argv)
{
int i;

	printf("ttpal v%s starting\n",VERSION);

	getcookie(_VDO,&cookie);

	if (cookie != TT_VIDEO) {
		fprintf(stderr,"Not a TT!\n");
		quit(1);
	}

	if (argc < 2) {
		fprintf(stderr,"No registers specified!\n");
		quit(1);
	}

	for (i = 1; i < argc; i++) {
		testreg = atoi(argv[i]);
		if ((testreg < 0) || (testreg > 15))
			fprintf(stderr,"%d is not a valid STe register number\n\n",testreg);
		else Supexec(test);
	}

	quit(0);
	return 0;
}

void testpal(char *name,volatile WORD *palette,WORD newcolour)
{
WORD oldste, oldtt, newste, newtt, ttexpected;

	oldste = ste_palette[testreg];
	oldtt = tt_palette[testreg];
	palette[testreg] = newcolour;
	newste = ste_palette[testreg];
	newtt = tt_palette[testreg];
	ste_palette[testreg] = oldste;
	tt_palette[testreg] = oldtt;

	printf("Old palette register values: STe[%d]=0x%04x TT[%d]=0x%04x\n",
			testreg,oldste,testreg,oldtt);
	printf("Set %s[%d] to 0x%04x\n",name,testreg,newcolour);
	printf("New palette register values: STe[%d]=0x%04x TT[%d]=0x%04x\n",
			testreg,newste,testreg,newtt);
	ttexpected = ((newste&0x0777)<<1) | ((newste&0x0888)>>3);
	if (newtt != ttexpected)
		printf("*** Palette register %d not synchronised ***\n",testreg);
	printf("(Restore original palette register values)\n");
}

long test()
{
	testpal("STe",ste_palette,STE_YELLOW);
	testpal("TT",tt_palette,TT_BLUE);
	printf("\n");

	return 0;
}

void quit(int rc)
{
	printf("Press a key to exit\n");
	getchar();
	exit(rc);
}
